#include "defines.h"

// Get the external gluObj object
extern GLUquadricObj* gluObj;

// Joint rotation
extern int currentjoint;
extern int joints[NUMJOINTS][3];

// Function that rotates the joint number specified by i
void movit(int i)
{
	// Makes it flash when you select it
	if(i == currentjoint)
	{
		// This won't work on Win32 :(
		if(time(NULL)%2)
			setMaterials(&Emerald, GL_FRONT_AND_BACK);
		else
			setMaterials(&Turquoise, GL_FRONT_AND_BACK);
	}

	// Limitations on the physics (so the body doesn't go into itself)
	if(joints[i][0] < -45) joints[i][0] = -45;
	else if(joints[i][0] > 45) joints[i][0] = 45;

	if(joints[i][1] < -30) joints[i][1]=-30;
	else if(joints[i][1]>30) joints[i][1]=30;

	if(joints[i][2] < -30) joints[i][2]=-30;
	else if(joints[i][2]>30) joints[i][2]=30;

	// Rotates the X, Y, and Z axes of the joints
	glRotatef(joints[i][0], 1.0, 0.0, 0.0);
	glRotatef(joints[i][1], 0.0, 1.0, 0.0);
	glRotatef(joints[i][2], 0.0, 0.0, 1.0);
}

// Creates the bug, frontbody and backbody
void bug(void)
{
	// Sets the initial material to brass, since we have a brass bug :)
	setMaterials(&Brass, GL_FRONT_AND_BACK);

	glPushMatrix();
		// Draws the front body
		frontbody();
	glPopMatrix();

	glPushMatrix();
		// Draws the back body
		backbody();
	glPopMatrix();
}

// Draws the front body
void frontbody(void)
{
	// Variables to make the spines
	GLfloat inc;
	int flag = 1;

	// Moves the front torso via user input
	movit(0);

	// Initially we're facing +z axis, so turn to the right
	glRotatef(90, 0.0, 1.0, 0.0);
	gluCylinder(gluObj, 0.06, 0.07, 0.45, 16, 16);

	// Make some hairy stuff 0.01 units apart, going toward the front
	for(inc=0.05;inc<0.3;inc+=0.01)
	{
		glPushMatrix();
			// Copper spines
			setMaterials(&Copper, GL_FRONT_AND_BACK);
			if(flag == 0)
			{
				glRotatef(10, 0.0, 0.0, 1.0);
				glTranslatef(0.005, 0.18, inc);
				flag++;
			}
			else if(flag == 1)
			{
				glRotatef(-10, 0.0, 0.0, 1.0);
				glTranslatef(-0.005, 0.18, inc);
				flag++;
			}
			else if(flag == 2)
			{
				glRotatef(-20, 0.0, 0.0, 1.0);
				glTranslatef(0.01, 0.18, inc);
				flag++;
			}
			else if(flag == 3)
			{
				glRotatef(-20, 0.0, 0.0, 1.0);
				glTranslatef(-0.01, 0.18, inc);
				flag=0;
			}
			glRotatef(90, 1.0, 0.0, 0.0);
			gluCylinder(gluObj, 0.001, 0.005, 0.12, 8, 8);
		glPopMatrix();
	}

	// Head
	setMaterials(&Brass, GL_FRONT_AND_BACK);
	glTranslatef(0.0, 0.0, 0.4);
	gluSphere(gluObj, 0.1, 10, 10);
	glPushMatrix();
		movit(25);
		neck();
	glPopMatrix();
	glTranslatef(0.0, 0.0, -0.4);

	// Three legs facing us, some 0.08 units apart
	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.08*1);
		legl(1);
		bottomleg(2);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.08*2);
		legl(3);
		bottomleg(4);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.08*3);
		legl(5);
		bottomleg(6);
	glPopMatrix();

	// Three legs away from us, some 0.08 units apart
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.08*1);
		legr(7);
		bottomleg(8);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.08*2);
		legr(9);
		bottomleg(10);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.08*3);
		legr(11);
		bottomleg(12);
	glPopMatrix();
}

// Draws the top portion of the legs on the left
void legl(int n)
{
	// Moves the top part of the leg move via user input
	movit(n);

	glRotatef(-90, 0.0, 1.0, 0.0);
	glRotatef(24, 1.0, 0.0, 0.0);
	gluCylinder(gluObj, 0.01, 0.01, 0.1, 8, 8);

	// Make some more spiney things on the legs
	glPushMatrix();
		setMaterials(&Copper, GL_FRONT_AND_BACK);
		glTranslatef(0.0, 0.0, 0.1);
		glRotatef(-75, 1.0, 0.0, 0.0);
		gluCylinder(gluObj, 0.005, 0.001, 0.12, 8, 8);
		setMaterials(&Brass, GL_FRONT_AND_BACK);
	glPopMatrix();
}

// Draws the top portion of the legs on the right
void legr(int n)
{
	// Moves the top part of the leg move via user input
	movit(n);

	glRotatef(90, 0.0, 1.0, 0.0);
	glRotatef(24, 1.0, 0.0, 0.0);
	gluCylinder(gluObj, 0.01, 0.01, 0.1, 8, 8);

	// Make some more spiney things on the legs
	glPushMatrix();
		setMaterials(&Copper, GL_FRONT_AND_BACK);
		glTranslatef(0.0, 0.0, 0.1);
		glRotatef(-75, 1.0, 0.0, 0.0);
		gluCylinder(gluObj, 0.005, 0.001, 0.12, 8, 8);
		setMaterials(&Brass, GL_FRONT_AND_BACK);
	glPopMatrix();
}

// Drwas the bottom of the legs (same for right and left)
void bottomleg(int n)
{
	glPushMatrix();
		setMaterials(&Brass, GL_FRONT_AND_BACK);
		glTranslatef(0.0, 0.0, 0.1);

		// Moves according to the number
		movit(n);

		gluSphere(gluObj, 0.02, 8, 8);
		glRotatef(40, 1.0, 0.0, 0.0);
		gluCylinder(gluObj, 0.01, 0.01, 0.1, 8, 8);
		glTranslatef(0.0, 0.0, 0.1);

		// Little chrome balls on the bottom (hoofs?)
		setMaterials(&Chrome, GL_FRONT_AND_BACK);
		gluSphere(gluObj, 0.02, 8, 8);	// Hoofs
		setMaterials(&Brass, GL_FRONT_AND_BACK);
	glPopMatrix();
}

// The cylinder that connects the head to the front body
void neck(void)
{
	glRotatef(-45, 1.0, 0.0, 0.0);
	gluCylinder(gluObj, 0.03, 0.04, 0.3, 8, 8);
	glTranslatef(0.0, 0.0, 0.35);
	glPushMatrix();
		// Head and antenna will be "Pearl"
		setMaterials(&Pearl, GL_FRONT_AND_BACK);
		movit(26);
		head();
		setMaterials(&Brass, GL_FRONT_AND_BACK);
	glPopMatrix();
}

// Draws the head
void head(void)
{
	gluSphere(gluObj, 0.12, 24, 24);

	// Eyes
	glPushMatrix();
		setMaterials(&Pewter, GL_FRONT_AND_BACK);
		glRotatef(50, 1.0, 0.0, 0.0);
		glTranslatef(-0.07, 0.05, 0.1);
		gluSphere(gluObj, 0.02, 6, 6);

		// Pupils or whatever
		glPushMatrix();
			setMaterials(&Silver, GL_FRONT_AND_BACK);
			glTranslatef(-0.003, 0.0, 0.015);
			gluSphere(gluObj, 0.01, 4, 4);
		glPopMatrix();

		setMaterials(&Pearl, GL_FRONT_AND_BACK);
	glPopMatrix();

	// Eyes on right
	glPushMatrix();
		setMaterials(&Pewter, GL_FRONT_AND_BACK);
		glRotatef(50, 1.0, 0.0, 0.0);
		glTranslatef(0.07, 0.05, 0.1);
		gluSphere(gluObj, 0.02, 6, 6);

		// Pupils or whatever
		glPushMatrix();
			setMaterials(&Silver, GL_FRONT_AND_BACK);
			glTranslatef(0.003, 0.0, 0.015);
			gluSphere(gluObj, 0.01, 4, 4);
		glPopMatrix();

		setMaterials(&Pearl, GL_FRONT_AND_BACK);
	glPopMatrix();

	// left antennae
	glPushMatrix();
		setMaterials(&Pearl, GL_FRONT_AND_BACK);

		glRotatef(-20, 0.0, 1.0, 0.0);
		glRotatef(10, 0.0, 0.0, 1.0);

		glTranslatef(-0.05, 0.05, 0.1);
		glRotatef(-20, 1.0, 0.0, 0.0);
		gluCylinder(gluObj, 0.01, 0.01, 0.15, 4, 4);

		glTranslatef(0.0, 0.0, 0.15);

		// Moves the top portion of the antenna
		movit(27);
		glPushMatrix();
			glRotatef(70, 1.0, 0.0, 0.0);
			glRotatef(-4, 0.0, 1.0, 0.0);
			gluCylinder(gluObj, 0.01, 0.01, 0.15, 4, 4);
		glPopMatrix();
	glPopMatrix();

	// right antennae
	glPushMatrix();
		setMaterials(&Pearl, GL_FRONT_AND_BACK);

		glRotatef(20, 0.0, 1.0, 0.0);
		glRotatef(-10, 0.0, 0.0, 1.0);

		glTranslatef(0.05, 0.05, 0.1);
		glRotatef(-20, 1.0, 0.0, 0.0);
		gluCylinder(gluObj, 0.01, 0.01, 0.15, 4, 4);

		glTranslatef(0.0, 0.0, 0.15);

		// Moves the top portion of the antenna
		movit(28);
		glPushMatrix();
			glRotatef(70, 1.0, 0.0, 0.0);
			glRotatef(-4, 0.0, 1.0, 0.0);
			gluCylinder(gluObj, 0.01, 0.01, 0.15, 4, 4);
		glPopMatrix();
	glPopMatrix();
}

// Draws the back portion of the creature
void backbody(void)
{
	GLfloat inc;
	int flag = 0;

	// Initially we're facing +z axis, so turn to the right
	glRotatef(-90, 0.0, 1.0, 0.0);
	gluCylinder(gluObj, 0.06, 0.07, 0.45, 16, 16);

	// Make some hairy stuff, same as with the front body
	for(inc=0.05;inc<0.4;inc+=0.01)
	{
		glPushMatrix();
			setMaterials(&Copper, GL_FRONT_AND_BACK);
			if(flag == 0)
			{
				glRotatef(10, 0.0, 0.0, 1.0);
				glTranslatef(0.005, 0.18, inc);
				flag++;
			}
			else if(flag == 1)
			{
				glRotatef(-10, 0.0, 0.0, 1.0);
				glTranslatef(-0.005, 0.18, inc);
				flag++;
			}
			else if(flag == 2)
			{
				glRotatef(-20, 0.0, 0.0, 1.0);
				glTranslatef(0.01, 0.18, inc);
				flag++;
			}
			else if(flag == 3)
			{
				glRotatef(-20, 0.0, 0.0, 1.0);
				glTranslatef(-0.01, 0.18, inc);
				flag=0;
			}
			glRotatef(90, 1.0, 0.0, 0.0);
			gluCylinder(gluObj, 0.001, 0.005, 0.12, 4, 4);
		glPopMatrix();
	}

	// Three legs facing us, some 0.08 units apart
	setMaterials(&Brass, GL_FRONT_AND_BACK);

	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.1*1);
		legl(13);
		bottomleg(14);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.1*2);
		legl(15);
		bottomleg(16);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-0.05, 0.0, 0.1*3);
		legl(17);
		bottomleg(18);
	glPopMatrix();

	// Three legs away from us, some 0.08 units apart
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.1*1);
		legr(19);
		bottomleg(20);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.1*2);
		legr(21);
		bottomleg(22);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.05, 0.0, 0.1*3);
		legr(23);
		bottomleg(24);
	glPopMatrix();

	// Make the tail thingy
	glPushMatrix();
		glTranslatef(0.0, 0.0, 0.45);
		gluCylinder(gluObj, 0.07, 0.04, 0.1, 16, 16);
		glPushMatrix();
			glTranslatef(0.0, 0.0, 0.1);
			gluSphere(gluObj, 0.04, 8, 8);
		glPopMatrix();
	glPopMatrix();
}
